LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.blake_pak.ALL;

ENTITY blake_control IS
	PORT (
		clk_in : IN STD_LOGIC; 
		reset_in : IN STD_LOGIC;
		
		h_init_in : IN  STD_LOGIC_WIDTHx8;	
		m_init_in : IN  STD_LOGIC_WIDTHx16;	
		s_init_in : IN  STD_LOGIC_WIDTHx4;
		v_init_in : IN  STD_LOGIC_WIDTHx16;
		
		v_1g_in : IN  STD_LOGIC_WIDTHx16;

		h_out : OUT  STD_LOGIC_WIDTHx8;	
		m_out : OUT  STD_LOGIC_WIDTHx16;	
		s_out : OUT  STD_LOGIC_WIDTHx4;
		v_out : OUT  STD_LOGIC_WIDTHx16;		
		
		round_cnt_out : OUT  NATURAL;
		round_num_out : OUT  STD_LOGIC_VECTOR(2 DOWNTO 0);
		
		inEN : IN  STD_LOGIC;
		outEN : OUT  STD_LOGIC
	);
END blake_control;

ARCHITECTURE blake_control_a OF blake_control IS

	TYPE state IS (init, round, fin);
	SIGNAL state_reg, state_next : state;
	
	SIGNAL round_cnt_reg, round_cnt_next : NATURAL; --runde
	SIGNAL round_num_reg, round_num_next : STD_LOGIC_VECTOR(2 DOWNTO 0);	--runde x 8
	SIGNAL h_reg , h_next : STD_LOGIC_WIDTHx8;
	SIGNAL m_reg , m_next : STD_LOGIC_WIDTHx16;			
	SIGNAL s_reg , s_next : STD_LOGIC_WIDTHx4;				
	SIGNAL v_reg , v_next : STD_LOGIC_WIDTHx16;
	SIGNAL enable_reg , enable_next: STD_LOGIC;
BEGIN 
	
	PROCESS(clk_in, reset_in)
	BEGIN
		IF (reset_in = '1') THEN
			state_reg <= init;
		ELSIF (clk_in'EVENT AND clk_in = '1') THEN	
			state_reg <= state_next;
		END IF;
	END PROCESS;
	
	PROCESS (state_reg, inEN, round_cnt_reg, round_num_reg, h_reg, m_reg, s_reg, v_reg, v_init_in,s_init_in,m_init_in,h_init_in,v_1g_in)
	BEGIN
		enable_next <= '0';
		round_cnt_next <= round_cnt_reg;
		round_num_next <= round_num_reg;	
		h_next <= h_reg;
		m_next <= m_reg;			
		s_next <= s_reg;			
		v_next <= v_reg;
	
		CASE state_reg IS
			WHEN init =>
				IF inEN = '1' THEN
					state_next <= round;
					
					round_cnt_next <= 0;
					round_num_next <= "000";
					h_next <= h_init_in;
					m_next <= m_init_in;			
					s_next <= s_init_in;			
					v_next <= v_init_in;
				ELSE
					state_next <= init;
				END IF;
			WHEN round =>
				v_next <= v_1g_in;			
				IF (round_cnt_reg < Num_of_rounds-1) OR (NOT(round_num_reg="111")) THEN
					state_next <= round;
					
					round_num_next <= round_num_reg + 1;
					IF round_num_reg = "111" THEN
						round_cnt_next <= round_cnt_reg + 1;
					END IF;
				ELSE
					state_next <= fin;
					enable_next <= '1';
				END IF;
			WHEN fin =>
				state_next <= init;
			WHEN OTHERS =>
				state_next <= init;
		END CASE;
	END PROCESS;

	
	
	PROCESS(clk_in, reset_in)
	BEGIN
		IF (reset_in = '1') THEN
			h_reg <= INIT_WIDTHx8;
			m_reg <= INIT_WIDTHx16;			
			s_reg <= INIT_WIDTHx4;			
			v_reg <= INIT_WIDTHx16;
			enable_reg <= '0';
			round_cnt_reg <= 0;
			round_num_reg <= "000";			
		ELSIF (clk_in'EVENT AND clk_in = '1') THEN	
			h_reg <= h_next;
			m_reg <= m_next;			
			s_reg <= s_next;			
			v_reg <= v_next;
			enable_reg <= enable_next;
			round_cnt_reg <= round_cnt_next;
			round_num_reg <= round_num_next;				
		END IF;
	END PROCESS;

	-- PROCESS (round_cnt_reg, round_num_reg, state_reg, h_reg, m_reg, s_reg, v_reg, inEN ,v_init_in,s_init_in,m_init_in,h_init_in,v_1g_in)
	-- BEGIN
		-- enable_next <= '0';
		-- round_cnt_next <= round_cnt_reg;
		-- round_num_next <= round_num_reg;	
		-- h_next <= h_reg;
		-- m_next <= m_reg;			
		-- s_next <= s_reg;			
		-- v_next <= v_reg;
		-- CASE state_reg IS
			-- WHEN init =>
				-- IF inEN = '1' THEN
					-- round_cnt_next <= 0;
					-- round_num_next <= "000";
					-- h_next <= h_init_in;
					-- m_next <= m_init_in;			
					-- s_next <= s_init_in;			
					-- v_next <= v_init_in;
				-- END IF;
			-- WHEN round =>
				-- v_next <= v_1g_in;
				-- IF round_cnt_reg < Num_of_rounds-1 THEN
					
					-- round_num_next <= round_num_reg + 1;
						
					-- IF round_num_reg = "111" THEN
						-- round_cnt_next <= round_cnt_reg + 1;
					-- END IF;
					
				-- ELSE
					-- enable_next <= '1';
				-- END IF;
			-- WHEN OTHERS =>
		-- END CASE;
	-- END PROCESS;
	
	h_out <= h_reg ;
	m_out <= m_reg ;
	s_out <= s_reg ;
	v_out <= v_reg ;	
	
	round_cnt_out <= round_cnt_reg;
	round_num_out <= round_num_reg;	
	
	outEN <= enable_reg ;

END ARCHITECTURE;