LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.blake_pak.ALL;

ENTITY blake_fin IS
	PORT (
		clk_in : IN STD_LOGIC; 
		reset_in : IN STD_LOGIC;
		
		v_in : IN  STD_LOGIC_WIDTHx16;	
		h_in : IN  STD_LOGIC_WIDTHx8;	
		s_in : IN  STD_LOGIC_WIDTHx4;	

		hash_out : OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0)
	);
END blake_fin;




ARCHITECTURE blake_fin_a OF blake_fin IS
	SIGNAL hs, v : STD_LOGIC_WIDTHx8;	
BEGIN 

	hs(0) <= h_in(0) XOR s_in(0);
	hs(1) <= h_in(1) XOR s_in(1);	
	hs(2) <= h_in(2) XOR s_in(2);
	hs(3) <= h_in(3) XOR s_in(3);
	hs(4) <= h_in(4) XOR s_in(0);
	hs(5) <= h_in(5) XOR s_in(1);	
	hs(6) <= h_in(6) XOR s_in(2);	
	hs(7) <= h_in(7) XOR s_in(3);
	
	
	v(0) <= v_in(0) XOR v_in(8);
	v(1) <= v_in(1) XOR v_in(9);	
	v(2) <= v_in(2) XOR v_in(10);
	v(3) <= v_in(3) XOR v_in(11);
	v(4) <= v_in(4) XOR v_in(12);
	v(5) <= v_in(5) XOR v_in(13);	
	v(6) <= v_in(6) XOR v_in(14);	
	v(7) <= v_in(7) XOR v_in(15);
	
	hash_out(WIDTHx8-1 DOWNTO WIDTHx7) <= hs(0) XOR v(0); --promenjen raspored zbog ispisa
	hash_out(WIDTHx7-1 DOWNTO WIDTHx6) <= hs(1) XOR v(1);	
	hash_out(WIDTHx6-1 DOWNTO WIDTHx5) <= hs(2) XOR v(2);
	hash_out(WIDTHx5-1 DOWNTO WIDTHx4) <= hs(3) XOR v(3);
	hash_out(WIDTHx4-1 DOWNTO WIDTHx3) <= hs(4) XOR v(4);
	hash_out(WIDTHx3-1 DOWNTO WIDTHx2) <= hs(5) XOR v(5);	
	hash_out(WIDTHx2-1 DOWNTO WIDTHx1) <= hs(6) XOR v(6);	
	hash_out(WIDTHx1-1 DOWNTO 0)       <= hs(7) XOR v(7);
	
END ARCHITECTURE;