LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE WORK.blake_pak.ALL;

ENTITY wrapper_1g IS
    PORT (
      clk_i         : IN  STD_LOGIC; 
      reset_i       : IN  STD_LOGIC;

      data_i        : IN  STD_LOGIC_VECTOR (63 DOWNTO 0);
      data_select_i : IN  STD_LOGIC_VECTOR (11 DOWNTO 0);
      data_enable_i : IN  STD_LOGIC;
    
      hash_select_i : IN  STD_LOGIC_VECTOR (2  DOWNTO 0);
      hash_o        : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
      hash_ready_o  : OUT STD_LOGIC
    );   
END wrapper_1g;

ARCHITECTURE wrapper_1g_a OF wrapper_1g IS

	COMPONENT blake_1g IS
		PORT(
			clk   : IN  STD_LOGIC; 
			reset : IN  STD_LOGIC;
			h     : IN  STD_LOGIC_VECTOR (WIDTHx8-1  DOWNTO 0);
			m     : IN  STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
			s     : IN  STD_LOGIC_VECTOR (WIDTHx4-1  DOWNTO 0);
			t     : IN  STD_LOGIC_VECTOR (WIDTHx2-1  DOWNTO 0);
			
			inEN  : IN  STD_LOGIC;
			hash  : OUT STD_LOGIC_VECTOR (WIDTHx8-1  DOWNTO 0);
			outEN : OUT STD_LOGIC
		);
	END COMPONENT;

    
	CONSTANT INPUT_MEM_SIZE:  INTEGER := 15;
    CONSTANT OUTPUT_MEM_SIZE: INTEGER := 8;
 
    CONSTANT h_init_1 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"6A09E667BB67AE85";
    CONSTANT h_init_2 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"3C6EF372A54FF53A";
    CONSTANT h_init_3 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"510E527F9B05688C";
    CONSTANT h_init_4 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"1F83D9AB5BE0CD19";
    CONSTANT s_init_1 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"0000000000000000";
    CONSTANT s_init_2 :STD_LOGIC_VECTOR (63 DOWNTO 0) := x"0000000000000000";
 
	TYPE input_mem_data_type  IS ARRAY (0 TO INPUT_MEM_SIZE-1)  OF STD_LOGIC_VECTOR (63 DOWNTO 0);
	TYPE output_mem_data_type IS ARRAY (0 TO OUTPUT_MEM_SIZE-1) OF STD_LOGIC_VECTOR (31 DOWNTO 0);
    
    SIGNAL input_mem  :input_mem_data_type  := (OTHERS => (OTHERS => '0'));
    SIGNAL output_mem :output_mem_data_type := (OTHERS => (OTHERS => '0'));


    SIGNAL data_enable_i_reg :STD_LOGIC ;

    SIGNAL input_mem_next_0 :STD_LOGIC_VECTOR (63 DOWNTO 0);  
    SIGNAL input_mem_next_1 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_2 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_3 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_4 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_5 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_6 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_7 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    SIGNAL input_mem_next_8 :STD_LOGIC_VECTOR (63 DOWNTO 0);    
    
    SIGNAL h_input_data :STD_LOGIC_VECTOR (WIDTHx8-1  DOWNTO 0);
    SIGNAL m_input_data :STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
    SIGNAL s_input_data :STD_LOGIC_VECTOR (WIDTHx4-1  DOWNTO 0);
    SIGNAL t_input_data :STD_LOGIC_VECTOR (WIDTHx2-1  DOWNTO 0);

    SIGNAL blake_hash        :STD_LOGIC_VECTOR (WIDTHx8-1  DOWNTO 0);
    SIGNAL hash_ready_o_next :STD_LOGIC ;
BEGIN 

	blake_1g_com: blake_1g
	PORT MAP (
		clk => clk_i ,
		reset => reset_i ,
        
		h => h_input_data ,
		m => m_input_data ,
		s => s_input_data ,
		t => t_input_data ,
		inEN => data_enable_i_reg ,
		hash => blake_hash ,
		outEN => hash_ready_o_next 
	);

	PROCESS(clk_i, reset_i)
	BEGIN
		IF (reset_i = '1') THEN
            input_mem(0)  <= (OTHERS => '0');
            input_mem(1)  <= (OTHERS => '0');           
            input_mem(2)  <= (OTHERS => '0');
			input_mem(3)  <= (OTHERS => '0');
            input_mem(4)  <= (OTHERS => '0');
            input_mem(5)  <= (OTHERS => '0');           
            input_mem(6)  <= (OTHERS => '0');
		    input_mem(7)  <= (OTHERS => '0');        
            input_mem(8)  <= (OTHERS => '0');
            input_mem(9)  <= h_init_1;           
            input_mem(10) <= h_init_2;
		    input_mem(11) <= h_init_3;        
            input_mem(12) <= h_init_4;
            input_mem(13) <= s_init_1;           
            input_mem(14) <= s_init_2; 
			data_enable_i_reg <= '0';	     
        ELSIF (clk_i'EVENT AND clk_i = '1') THEN	
            input_mem(0)  <= input_mem_next_0;
            input_mem(1)  <= input_mem_next_1;           
            input_mem(2)  <= input_mem_next_2;
		    input_mem(3)  <= input_mem_next_3;
            input_mem(4)  <= input_mem_next_4;
            input_mem(5)  <= input_mem_next_5;           
            input_mem(6)  <= input_mem_next_6;
		    input_mem(7)  <= input_mem_next_7;        
            input_mem(8)  <= input_mem_next_8;
			input_mem(9)  <= h_init_1;           
            input_mem(10) <= h_init_2;
		    input_mem(11) <= h_init_3;        
            input_mem(12) <= h_init_4;
            input_mem(13) <= s_init_1;           
            input_mem(14) <= s_init_2; 	
            data_enable_i_reg <= data_enable_i;
		END IF;
	END PROCESS;
	
    input_mem_next_0  <= data_i when data_select_i = x"000" else input_mem(0);
    input_mem_next_1  <= data_i when data_select_i = x"001" else input_mem(1);           
    input_mem_next_2  <= data_i when data_select_i = x"002" else input_mem(2);
	input_mem_next_3  <= data_i when data_select_i = x"003" else input_mem(3);
    input_mem_next_4  <= data_i when data_select_i = x"004" else input_mem(4);
    input_mem_next_5  <= data_i when data_select_i = x"005" else input_mem(5);           
    input_mem_next_6  <= data_i when data_select_i = x"006" else input_mem(6);
	input_mem_next_7  <= data_i when data_select_i = x"007" else input_mem(7);        
    input_mem_next_8  <= data_i when data_select_i = x"008" else input_mem(8);  

    h_input_data <= input_mem(9) & input_mem(10) & input_mem(11) & input_mem(12);
	m_input_data <= input_mem(1) & input_mem(2) & input_mem(3) & input_mem(4) & input_mem(5) & input_mem(6) & input_mem(7) & input_mem(8);
	s_input_data <= input_mem(13) & input_mem(14);
	t_input_data <= input_mem(0);
    
	PROCESS(clk_i, reset_i)
	BEGIN
		IF (reset_i = '1') THEN
            output_mem(0)  <= (OTHERS => '0');
            output_mem(1)  <= (OTHERS => '0');           
            output_mem(2)  <= (OTHERS => '0');
		    output_mem(3)  <= (OTHERS => '0');
            output_mem(4)  <= (OTHERS => '0');
            output_mem(5)  <= (OTHERS => '0');           
            output_mem(6)  <= (OTHERS => '0');
		    output_mem(7)  <= (OTHERS => '0');  
            hash_ready_o   <= '0';		
        ELSIF (clk_i'EVENT AND clk_i = '1') THEN	
            output_mem(0)  <= blake_hash(31 DOWNTO 0);
            output_mem(1)  <= blake_hash(63 DOWNTO 32);       
            output_mem(2)  <= blake_hash(95 DOWNTO 64);
		    output_mem(3)  <= blake_hash(127 DOWNTO 96);
            output_mem(4)  <= blake_hash(159 DOWNTO 128);
            output_mem(5)  <= blake_hash(191 DOWNTO 160);           
            output_mem(6)  <= blake_hash(223 DOWNTO 192);
		    output_mem(7)  <= blake_hash(255 DOWNTO 224);        

            hash_ready_o   <= hash_ready_o_next;
		END IF;
	END PROCESS;    
    
    hash_o  <= output_mem(0) when hash_select_i = "000" else 
               output_mem(1) when hash_select_i = "001" else                
               output_mem(2) when hash_select_i = "010" else
               output_mem(3) when hash_select_i = "011" else               
               output_mem(4) when hash_select_i = "100" else               
               output_mem(5) when hash_select_i = "101" else               
               output_mem(6) when hash_select_i = "110" else               
               output_mem(7) ;             
 
	
END ARCHITECTURE ;