LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE WORK.blade_pak.ALL;
--USE WORK.blake256Pkg.ALL;

ENTITY testbench_1g IS
END testbench_1g;

----------------------------------------------------------------------------------------	
ARCHITECTURE testbench_1g_a OF testbench_1g IS

	COMPONENT blade_1g IS
		PORT(
			clk : IN STD_LOGIC; 
			reset : IN STD_LOGIC;
			h : IN  STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
			m : IN  STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
			s : IN  STD_LOGIC_VECTOR (WIDTHx4-1 DOWNTO 0);
			t : IN  STD_LOGIC_VECTOR (WIDTHx2-1 DOWNTO 0);
			
			inEN : IN  STD_LOGIC;
			hash  : OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
			outEN : OUT  STD_LOGIC
		);
	END COMPONENT;

	SIGNAL test_clk:  std_logic := 'L'; 
	SIGNAL test_reset:  std_logic := 'L';		
	SIGNAL test_1g_h : STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0) :=(OTHERS => 'L');
	SIGNAL test_1g_m : STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0):=(OTHERS => 'L');
	SIGNAL test_1g_s : STD_LOGIC_VECTOR (WIDTHx4-1 DOWNTO 0) :=(OTHERS => 'L');
	SIGNAL test_1g_t : STD_LOGIC_VECTOR (WIDTHx2-1 DOWNTO 0) :=(OTHERS => 'L');
	SIGNAL test_1g_inEN :   STD_LOGIC:= 'L';
	SIGNAL test_1g_hash  :  STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0) :=(OTHERS => 'L'); 
	SIGNAL test_1g_outEN :  STD_LOGIC:= 'L';

	
	CONSTANT T : time := 20ns;
	SIGNAL monitor_num : NATURAL:= 0;
	SIGNAL the_end : STD_LOGIC:= '0';	

	CONSTANT test_vector_num: INTEGER := 4;
	
	TYPE h_mem_data_type   IS ARRAY (0 TO test_vector_num-1) OF STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
	TYPE m_mem_data_type   IS ARRAY (0 TO test_vector_num-1) OF STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
	TYPE s_mem_data_type   IS ARRAY (0 TO test_vector_num-1) OF STD_LOGIC_VECTOR (WIDTHx4-1 DOWNTO 0);
	TYPE t_mem_data_type   IS ARRAY (0 TO test_vector_num-1) OF STD_LOGIC_VECTOR (WIDTHx2-1 DOWNTO 0);
	
	CONSTANT h_mem :h_mem_data_type := ((x"6A09E667BB67AE853C6EF372A54FF53A510E527F9B05688C1F83D9AB5BE0CD19"),
										(x"6A09E667BB67AE853C6EF372A54FF53A510E527F9B05688C1F83D9AB5BE0CD19"),
										(OTHERS => '0'),
										(OTHERS => '0')
									  );
	CONSTANT m_mem :m_mem_data_type := ((x"00800000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000010000000000000008"),
										(x"00800000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000010000000000000008"),
										(OTHERS => '0'),
										(OTHERS => '0')
									  );
	CONSTANT s_mem :s_mem_data_type := ((x"00000000000000000000000000000000"),
										(x"00000000000000000000000000000000"),
										(OTHERS => '0'),
										(OTHERS => '0')
									  );
	CONSTANT t_mem :t_mem_data_type := ((x"0000000800000000"),
										(x"0000000800000000"),
										(OTHERS => '0'),
										(OTHERS => '0')
									  );
	
	-- TYPE mem_data_type   IS ARRAY (0 TO 63) OF STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
	-- SIGNAL mem_pipe : mem_data_type;
	-- SIGNAL mem_1g  : mem_data_type;


	SIGNAL gen_enable_1g :  STD_LOGIC_VECTOR (test_vector_num-1 DOWNTO 0):= "0011";  --bit enable vektor
	SIGNAL bit_enable_num : NATURAL:= 2;-- broj jedinica u gen_enable_1g			------------------------!!!!!!!!!!!!!!!****
--****************************************************************************************************	
	
BEGIN 

	blade_1g_com: blade_1g
	PORT MAP (
		clk => test_clk ,
		reset => test_reset ,
		h => test_1g_h ,
		m => test_1g_m ,
		s => test_1g_s ,
		t => test_1g_t ,
		inEN => test_1g_inEN ,
		hash => test_1g_hash ,
		outEN => test_1g_outEN 
	);

--****************************************************************************************************
--generatori takta
--****************************************************************************************************
	PROCESS
	BEGIN
		test_clk <= '0';
		WAIT FOR T/2;
		test_clk <= '1';
		WAIT FOR T/2;
	END PROCESS;
	
	test_reset  <='1', '0' AFTER T/2;	
	
-- ****************************************************************************************************
-- pipe test vektor generator
-- ****************************************************************************************************
	
	-- PROCESS
		-- VARIABLE num_input	: INTEGER := 0;
		-- VARIABLE gen_num	: INTEGER := 0;		
	-- BEGIN
		-- WAIT UNTIL falling_edge (test_reset);
		-- WAIT UNTIL falling_edge (test_clk);
		
		-- FOR i IN 0 TO test_vector_num-1 LOOP 		--ovde mozemo da testiramo jak saobracaj samo pustimo jedan te isti patern da nam se heshira
			-- WAIT UNTIL falling_edge (test_clk);	
			
			-- test_pipe_h <=h_mem(num_input) ;
			-- test_pipe_m <=m_mem(num_input) ;
			-- test_pipe_s <=s_mem(num_input) ;
			-- test_pipe_t <=t_mem(num_input) ;
			
			-- IF(pipe_gen_enable(gen_num) = '1') THEN	
				-- num_input:=num_input + 1;
				-- test_pipe_inEN <= '1' ;
			-- ELSE
				-- test_pipe_inEN <= '0' ;
			-- END IF; 
			
			-- IF(gen_num=test_vector_num-1) THEN	gen_num:=0; ELSE gen_num:=gen_num + 1; END IF;
			
		-- END LOOP;
			
	-- END PROCESS;
		
--8g (autorski dizajn) test vektor generator (autorski dizajn sluzi logika koja ce generisati izlaze
-- koji ce se uporedjivari sa izlazima naseg pipeline dizajna i naj nacin testiracemo funkcionalnost)
--****************************************************************************************************
	
	PROCESS
		VARIABLE num_input	: INTEGER := 0;
		VARIABLE gen_num	: INTEGER := 0;		
	BEGIN
		WAIT UNTIL falling_edge (test_reset);
		WAIT UNTIL falling_edge (test_clk);
		
		FOR i IN 0 TO test_vector_num-1 LOOP 		--ovde mozemo da testiramo jak saobracaj samo pustimo jedan te isti patern da nam se heshira
			WAIT UNTIL falling_edge (test_clk);	
			
			test_1g_h <=h_mem(num_input) ;
			test_1g_m <=m_mem(num_input) ;
			test_1g_s <=s_mem(num_input) ;
			test_1g_t <=t_mem(num_input) ;
			
			IF(gen_enable_1g(gen_num) = '1') THEN	
				num_input:=num_input + 1;
				test_1g_inEN <= '1' ;
			ELSE
				test_1g_inEN <= '0' ;
			END IF; 
			
			IF(gen_num=test_vector_num-1) THEN	gen_num:=0; ELSE gen_num:=gen_num + 1; END IF;
			
			WAIT UNTIL test_1g_outEN='1';
			
		END LOOP;
			
	END PROCESS;
	
--****************************************************************************************************
--scoreboard and monitor
--****************************************************************************************************
	-- PROCESS (test_clk, test_reset)
		-- VARIABLE pipe_num	: INTEGER := 0;
		-- VARIABLE org_num	: INTEGER := 0;			
	-- BEGIN

		-- IF (test_reset = '1') THEN
			-- FOR i IN 0 TO 63 LOOP 
				-- mem_pipe(i)<=(OTHERS => '0');
				-- mem_org(i)<=(OTHERS => '0');
			-- END LOOP;
		-- ELSIF (test_clk'EVENT and test_clk = '1') THEN
			
			-- IF (test_pipe_outEN='1') THEN
				-- mem_pipe(pipe_num)<=test_pipe_hash;
				-- pipe_num:=pipe_num+1;
			-- END IF;

			-- IF (test_org_outEN='1') THEN
				-- mem_org(org_num)<=test_org_hash;
				-- org_num:=org_num+1;
			-- END IF;
		
			-- IF (pipe_num=bit_enable_num AND org_num=bit_enable_num) THEN --procitali sve validne ulaze iz vektora, !!!! saljemo npr 10 uzastopnih vektora i ovde mnozimo sa 10
				-- the_end<='1';
			-- END IF;

			
		-- END IF;	
	-- END PROCESS;
	
--****************************************************************************************************
--checker and the end
--****************************************************************************************************	
	-- PROCESS -- kraj
		-- VARIABLE monitor_num	: INTEGER := 0;
	-- BEGIN
		-- WAIT UNTIL the_end='1';
		
		-- FOR i IN 0 TO 63 LOOP 
			-- IF (mem_pipe(monitor_num)=mem_org(monitor_num)) THEN 
				-- ASSERT false
					-- REPORT "nepoklapanje rezultata"	
				-- SEVERITY NOTE;
			-- END IF;
			-- monitor_num:=monitor_num+1;
		-- END LOOP;
		
		-- WAIT UNTIL falling_edge (test_clk);
		
		-- ASSERT false
			-- REPORT "simulation completed"	
		-- SEVERITY NOTE;
		
	-- END PROCESS;
	
END ARCHITECTURE ;