LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE WORK.blake_pak.ALL;

ENTITY blake_112g IS
    PORT (
        clk : IN STD_LOGIC; 
        reset : IN STD_LOGIC;
        
        h : IN  STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
        m : IN  STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
        s : IN  STD_LOGIC_VECTOR (WIDTHx4-1 DOWNTO 0);
        t : IN  STD_LOGIC_VECTOR (WIDTHx2-1 DOWNTO 0);
        inEN : IN  STD_LOGIC;
        
        hash  : OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
        outEN : OUT  STD_LOGIC
    );
END blake_112g;

ARCHITECTURE blake_112g_a OF blake_112g IS

    COMPONENT blake_init IS
        PORT (
            clk_in : IN STD_LOGIC; 
            reset_in : IN STD_LOGIC;
            
            h_in : IN STD_LOGIC_WIDTHx8;
            s_in : IN STD_LOGIC_WIDTHx4;
            t_in : IN  STD_LOGIC_WIDTHx2;
            
            v_out : OUT STD_LOGIC_WIDTHx16
        );
    END COMPONENT;  

    COMPONENT blake_control IS
        PORT (
            clk_in : IN STD_LOGIC; 
            reset_in : IN STD_LOGIC;
            
            inEN : IN  STD_LOGIC;
            outEN : OUT  STD_LOGIC
        );
    END COMPONENT;  
    
    COMPONENT round_8g IS
        GENERIC(
            round_cnt_in :  NATURAL
        );
        PORT (
            clk_in : IN STD_LOGIC; 
            reset_in : IN STD_LOGIC;
            
            
            m_in : IN STD_LOGIC_WIDTHx16;
            v_in : IN STD_LOGIC_WIDTHx16;
            
            v_out : OUT STD_LOGIC_WIDTHx16
		);
    END COMPONENT;  

    COMPONENT blake_fin IS
        PORT (
            clk_in : IN STD_LOGIC; 
            reset_in : IN STD_LOGIC;
            
            v_in : IN  STD_LOGIC_WIDTHx16;  
            h_in : IN  STD_LOGIC_WIDTHx8;   
            s_in : IN  STD_LOGIC_WIDTHx4;   

            hash_out : OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0)
        );
    END COMPONENT;  

	SIGNAL v_init         : STD_LOGIC_WIDTHx16;
	SIGNAL v1_out_v2_in   : STD_LOGIC_WIDTHx16;    
	SIGNAL v2_out_v3_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v3_out_v4_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v4_out_v5_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v5_out_v6_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v6_out_v7_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v7_out_v8_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v8_out_v9_in   : STD_LOGIC_WIDTHx16;        
	SIGNAL v9_out_v10_in  : STD_LOGIC_WIDTHx16;        
 	SIGNAL v10_out_v11_in : STD_LOGIC_WIDTHx16;       
	SIGNAL v11_out_v12_in : STD_LOGIC_WIDTHx16;        
	SIGNAL v12_out_v13_in : STD_LOGIC_WIDTHx16;        
	SIGNAL v13_out_v14_in : STD_LOGIC_WIDTHx16;          
	SIGNAL v14_out        : STD_LOGIC_WIDTHx16; 

	SIGNAL h_array : STD_LOGIC_WIDTHx8;
	SIGNAL m_array : STD_LOGIC_WIDTHx16;
	SIGNAL s_array : STD_LOGIC_WIDTHx4;
	SIGNAL t_array : STD_LOGIC_WIDTHx2;


BEGIN 
-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------
	h_array(7) <= h(WIDTHx1-1 DOWNTO 0);        --konverzija iz velikog vektora u niz manjih
	h_array(6) <= h(WIDTHx2-1 DOWNTO WIDTHx1); --!!!!!!!!!! BITNA STVAR REDOSLED DODELJIVANJA IZ RECI U NIZ!!! OBRNUTO OD OCEKIVANOG,RAZLOG PATERNI IZ LIRERATURE
	h_array(5) <= h(WIDTHx3-1 DOWNTO WIDTHx2);
	h_array(4) <= h(WIDTHx4-1 DOWNTO WIDTHx3);
	h_array(3) <= h(WIDTHx5-1 DOWNTO WIDTHx4); 
	h_array(2) <= h(WIDTHx6-1 DOWNTO WIDTHx5); 
	h_array(1) <= h(WIDTHx7-1 DOWNTO WIDTHx6); 
	h_array(0) <= h(WIDTHx8-1 DOWNTO WIDTHx7); 

	m_array(15)<= m(WIDTHx1-1  DOWNTO 0);
	m_array(14)<= m(WIDTHx2-1  DOWNTO WIDTHx1);  
	m_array(13)<= m(WIDTHx3-1  DOWNTO WIDTHx2);  
	m_array(12)<= m(WIDTHx4-1  DOWNTO WIDTHx3); 
	m_array(11)<= m(WIDTHx5-1  DOWNTO WIDTHx4);  
	m_array(10)<= m(WIDTHx6-1  DOWNTO WIDTHx5); 
	m_array(9) <= m(WIDTHx7-1  DOWNTO WIDTHx6); 
	m_array(8) <= m(WIDTHx8-1  DOWNTO WIDTHx7);  
	m_array(7) <= m(WIDTHx9-1  DOWNTO WIDTHx8);  
	m_array(6) <= m(WIDTHx10-1 DOWNTO WIDTHx9);  
	m_array(5) <= m(WIDTHx11-1 DOWNTO WIDTHx10);  
	m_array(4) <= m(WIDTHx12-1 DOWNTO WIDTHx11); 
	m_array(3) <= m(WIDTHx13-1 DOWNTO WIDTHx12); 
	m_array(2) <= m(WIDTHx14-1 DOWNTO WIDTHx13);  
	m_array(1) <= m(WIDTHx15-1 DOWNTO WIDTHx14);  
	m_array(0) <= m(WIDTHx16-1 DOWNTO WIDTHx15);  

	s_array(3) <= s(WIDTHx1-1 DOWNTO 0);
	s_array(2) <= s(WIDTHx2-1 DOWNTO WIDTHx1);  
	s_array(1) <= s(WIDTHx3-1 DOWNTO WIDTHx2);  
	s_array(0) <= s(WIDTHx4-1 DOWNTO WIDTHx3); 
		
	t_array(1) <= t(WIDTHx1-1 DOWNTO 0);
	t_array(0) <= t(WIDTHx2-1 DOWNTO WIDTHx1);  
-----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------
	
	blake_init_com: blake_init 
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		h_in => h_array ,
		s_in => s_array ,
		t_in => t_array ,
		
		v_out => v_init 
	);

	blake_control_com: blake_control 
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,

		inEN => inEN ,
		outEN => outEN 
	);
	
	round_8g_1_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 0
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v_init ,
		
		v_out =>  v1_out_v2_in
	);
    
    round_8g_2_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 1
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v1_out_v2_in ,
		
		v_out =>  v2_out_v3_in
	);
 
    
    round_8g_3_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 2
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v2_out_v3_in ,
		
		v_out =>  v3_out_v4_in
	);

    
    round_8g_4_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 3
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v3_out_v4_in ,
		
		v_out =>  v4_out_v5_in
	);

    
    round_8g_5_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 4
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v4_out_v5_in ,
		
		v_out =>  v5_out_v6_in
	);

    
    round_8g_6_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 5
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v5_out_v6_in ,
		
		v_out =>  v6_out_v7_in
	);

    
    round_8g_7_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 6
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v6_out_v7_in ,
		
		v_out =>  v7_out_v8_in
	);

    
    round_8g_8_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 7
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v7_out_v8_in ,
		
		v_out =>  v8_out_v9_in
	);

    
    round_8g_9_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 8
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v8_out_v9_in ,
		
		v_out =>  v9_out_v10_in
	);

    
    round_8g_10_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 9
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v9_out_v10_in ,
		
		v_out =>  v10_out_v11_in
	);

    
    round_8g_11_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 10
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v10_out_v11_in ,
		
		v_out =>  v11_out_v12_in
	);

    
    round_8g_12_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 11
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v11_out_v12_in ,
		
		v_out =>  v12_out_v13_in
	);

    
    round_8g_13_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 12
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v12_out_v13_in ,
		
		v_out =>  v13_out_v14_in
	); 

    
    round_8g_14_com: round_8g 
    GENERIC MAP (
		round_cnt_in => 13
	)
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		m_in => m_array ,
		v_in => v13_out_v14_in ,
		
		v_out =>  v14_out
	);    
 
	blake_fin_com: blake_fin 
	PORT MAP (
		clk_in => clk ,
		reset_in => reset ,
		
		v_in => v14_out ,
		h_in => h_array ,
		s_in => s_array ,

		hash_out => hash 
	);  

			
END ARCHITECTURE;