LIBRARY ieee;
USE ieee.STD_LOGIC_1164.ALL;

PACKAGE blake_pak IS

	CONSTANT Num_of_rounds : integer := 14; --Broj rundi u dizajnu
	CONSTANT Num_of_stages : integer := 2*Num_of_rounds +2; --Broj stepeni u pipeline-u

	CONSTANT WIDTHx1  : integer := 32;
	CONSTANT WIDTHx2  : integer := 2*WIDTHx1;
	CONSTANT WIDTHx3  : integer := 3*WIDTHx1;
	CONSTANT WIDTHx4  : integer := 4*WIDTHx1;
	CONSTANT WIDTHx5  : integer := 5*WIDTHx1;
	CONSTANT WIDTHx6  : integer := 6*WIDTHx1;
	CONSTANT WIDTHx7  : integer := 7*WIDTHx1;
	CONSTANT WIDTHx8  : integer := 8*WIDTHx1;
	CONSTANT WIDTHx9  : integer := 9*WIDTHx1;
	CONSTANT WIDTHx10 : integer := 10*WIDTHx1;
	CONSTANT WIDTHx11 : integer := 11*WIDTHx1;
	CONSTANT WIDTHx12 : integer := 12*WIDTHx1;
	CONSTANT WIDTHx13 : integer := 13*WIDTHx1;
	CONSTANT WIDTHx14 : integer := 14*WIDTHx1; 
	CONSTANT WIDTHx15 : integer := 15*WIDTHx1; 
	CONSTANT WIDTHx16 : integer := 16*WIDTHx1;
												
	CONSTANT IV0 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"6A09E667";
	CONSTANT IV1 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"BB67AE85";
	CONSTANT IV2 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"3C6EF372";
	CONSTANT IV3 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"A54FF53A";
	CONSTANT IV4 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"510E527F";
	CONSTANT IV5 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"9B05688C";
	CONSTANT IV6 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"1F83D9AB";
	CONSTANT IV7 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"5BE0CD19";

	CONSTANT C_CON_0  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"243F6A88";
	CONSTANT C_CON_1  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"85A308D3";
	CONSTANT C_CON_2  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"13198A2E";
	CONSTANT C_CON_3  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"03707344";
	CONSTANT C_CON_4  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"A4093822";
	CONSTANT C_CON_5  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"299F31D0";
	CONSTANT C_CON_6  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"082EFA98";
	CONSTANT C_CON_7  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"EC4E6C89";
	CONSTANT C_CON_8  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"452821E6";
	CONSTANT C_CON_9  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"38D01377";
	CONSTANT C_CON_10 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"BE5466CF";
	CONSTANT C_CON_11 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"34E90C6C";
	CONSTANT C_CON_12 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"C0AC29B7";
	CONSTANT C_CON_13 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"C97C50DD";
	CONSTANT C_CON_14 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"3F84D5B5";
	CONSTANT C_CON_15 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0) := x"B5470917";

	CONSTANT ROT1 : INTEGER := 16;
	CONSTANT ROT2 : INTEGER := 12;
	CONSTANT ROT3 : INTEGER := 8;
	CONSTANT ROT4 : INTEGER := 7;

	-- odavde sve isto za sve runde

	TYPE STD_LOGIC_WIDTHx2  IS ARRAY (0 TO 1)  OF  STD_LOGIC_VECTOR (WIDTHx1-1 DOWNTO 0);
	TYPE STD_LOGIC_WIDTHx4  IS ARRAY (0 TO 3)  OF  STD_LOGIC_VECTOR (WIDTHx1-1 DOWNTO 0);
	TYPE STD_LOGIC_WIDTHx8  IS ARRAY (0 TO 7)  OF  STD_LOGIC_VECTOR (WIDTHx1-1 DOWNTO 0);
	TYPE STD_LOGIC_WIDTHx16 IS ARRAY (0 TO 15) OF  STD_LOGIC_VECTOR (WIDTHx1-1 DOWNTO 0);

	CONSTANT INIT_WIDTHx2:  STD_LOGIC_WIDTHx2 :=((others=> (others=>'0')));
	CONSTANT INIT_WIDTHx4:  STD_LOGIC_WIDTHx4 :=((others=> (others=>'0')));
	CONSTANT INIT_WIDTHx8:  STD_LOGIC_WIDTHx8 :=((others=> (others=>'0')));                                 
	CONSTANT INIT_WIDTHx16: STD_LOGIC_WIDTHx16:=((others=> (others=>'0')));


	TYPE num IS ARRAY (0 TO 15) OF INTEGER;
	TYPE round_num IS ARRAY (0 TO 15) OF num;

	CONSTANT m_num: round_num:= ( (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15),
								  (14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3),
								  (11, 8, 12, 0, 5, 2, 15, 13, 10, 14, 3, 6, 7, 1, 9, 4),
								  (7, 9, 3, 1, 13, 12, 11, 14, 2, 6, 5, 10, 4, 0, 15, 8),
								  (9, 0, 5, 7, 2, 4, 10, 15, 14, 1, 11, 12, 6, 8, 3, 13),
								  (2, 12, 6, 10, 0, 11, 8, 3, 4, 13, 7, 5, 15, 14, 1, 9),
								  (12, 5, 1, 15, 14, 13, 4, 10, 0, 7, 6, 3, 9, 2, 8, 11),
								  (13, 11, 7, 14, 12, 1, 3, 9, 5, 0, 15, 4, 8, 6, 2, 10),
								  (6, 15, 14, 9, 11, 3, 0, 8, 12, 2, 13, 7, 1, 4, 10, 5),
								  (10, 2, 8, 4, 7, 6, 1, 5, 15, 11, 9, 14, 3, 12, 13, 0),
								  
								  (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15),
								  (14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3),
								  (11, 8, 12, 0, 5, 2, 15, 13, 10, 14, 3, 6, 7, 1, 9, 4),
								  (7, 9, 3, 1, 13, 12, 11, 14, 2, 6, 5, 10, 4, 0, 15, 8),
								  (9, 0, 5, 7, 2, 4, 10, 15, 14, 1, 11, 12, 6, 8, 3, 13),
								  (2, 12, 6, 10, 0, 11, 8, 3, 4, 13, 7, 5, 15, 14, 1, 9)
								);

									
	TYPE con IS ARRAY (0 TO 15) OF  STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	TYPE round_con IS ARRAY (0 TO 15) OF con;                               
									
	CONSTANT c_con: round_con:= ( (C_CON_1, C_CON_0, C_CON_3, C_CON_2, C_CON_5, C_CON_4, C_CON_7, C_CON_6, C_CON_9, C_CON_8, C_CON_11, C_CON_10, C_CON_13, C_CON_12, C_CON_15, C_CON_14),
								  (C_CON_10, C_CON_14, C_CON_8, C_CON_4, C_CON_15, C_CON_9, C_CON_6, C_CON_13, C_CON_12, C_CON_1, C_CON_2, C_CON_0, C_CON_7, C_CON_11, C_CON_3, C_CON_5),
								  (C_CON_8, C_CON_11, C_CON_0, C_CON_12, C_CON_2, C_CON_5, C_CON_13, C_CON_15, C_CON_14, C_CON_10, C_CON_6, C_CON_3, C_CON_1, C_CON_7, C_CON_4, C_CON_9),
								  (C_CON_9, C_CON_7, C_CON_1, C_CON_3, C_CON_12, C_CON_13, C_CON_14, C_CON_11, C_CON_6, C_CON_2, C_CON_10, C_CON_5, C_CON_0, C_CON_4, C_CON_8, C_CON_15),
								  (C_CON_0, C_CON_9, C_CON_7, C_CON_5, C_CON_4, C_CON_2, C_CON_15, C_CON_10, C_CON_1, C_CON_14, C_CON_12, C_CON_11, C_CON_8, C_CON_6, C_CON_13, C_CON_3),
								  (C_CON_12, C_CON_2, C_CON_10, C_CON_6, C_CON_11, C_CON_0, C_CON_3, C_CON_8, C_CON_13, C_CON_4, C_CON_5, C_CON_7, C_CON_14, C_CON_15, C_CON_9, C_CON_1),
								  (C_CON_5, C_CON_12, C_CON_15, C_CON_1, C_CON_13, C_CON_14, C_CON_10, C_CON_4, C_CON_7, C_CON_0, C_CON_3, C_CON_6, C_CON_2, C_CON_9, C_CON_11, C_CON_8),
								  (C_CON_11, C_CON_13, C_CON_14, C_CON_7, C_CON_1, C_CON_12, C_CON_9, C_CON_3, C_CON_0, C_CON_5, C_CON_4, C_CON_15, C_CON_6, C_CON_8, C_CON_10, C_CON_2),
								  (C_CON_15, C_CON_6, C_CON_9, C_CON_14, C_CON_3, C_CON_11, C_CON_8, C_CON_0, C_CON_2, C_CON_12, C_CON_7, C_CON_13, C_CON_4, C_CON_1, C_CON_5, C_CON_10),
								  (C_CON_2, C_CON_10, C_CON_4, C_CON_8, C_CON_6, C_CON_7, C_CON_5, C_CON_1, C_CON_11, C_CON_15, C_CON_14, C_CON_9, C_CON_12, C_CON_3, C_CON_0, C_CON_13),
								  
								  (C_CON_1, C_CON_0, C_CON_3, C_CON_2, C_CON_5, C_CON_4, C_CON_7, C_CON_6, C_CON_9, C_CON_8, C_CON_11, C_CON_10, C_CON_13, C_CON_12, C_CON_15, C_CON_14),
								  (C_CON_10, C_CON_14, C_CON_8, C_CON_4, C_CON_15, C_CON_9, C_CON_6, C_CON_13, C_CON_12, C_CON_1, C_CON_2, C_CON_0, C_CON_7, C_CON_11, C_CON_3, C_CON_5),
								  (C_CON_8, C_CON_11, C_CON_0, C_CON_12, C_CON_2, C_CON_5, C_CON_13, C_CON_15, C_CON_14, C_CON_10, C_CON_6, C_CON_3, C_CON_1, C_CON_7, C_CON_4, C_CON_9),
								  (C_CON_9, C_CON_7, C_CON_1, C_CON_3, C_CON_12, C_CON_13, C_CON_14, C_CON_11, C_CON_6, C_CON_2, C_CON_10, C_CON_5, C_CON_0, C_CON_4, C_CON_8, C_CON_15),
								  (C_CON_0, C_CON_9, C_CON_7, C_CON_5, C_CON_4, C_CON_2, C_CON_15, C_CON_10, C_CON_1, C_CON_14, C_CON_12, C_CON_11, C_CON_8, C_CON_6, C_CON_13, C_CON_3),
								  (C_CON_12, C_CON_2, C_CON_10, C_CON_6, C_CON_11, C_CON_0, C_CON_3, C_CON_8, C_CON_13, C_CON_4, C_CON_5, C_CON_7, C_CON_14, C_CON_15, C_CON_9, C_CON_1)
								);

END blake_pak;