LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
--USE IEEE.NUMERIC_STD.ALL;
--USE IEEE.STD_LOGIC_SIGNED.ALL;
--USE IEEE.STD_LOGIC_ARITH.ALL;
USE work.blake_pak.ALL;

ENTITY blake_fin IS
	PORT (
		clk_in : IN STD_LOGIC; 
		reset_in : IN STD_LOGIC;
		
		v_in : IN  STD_LOGIC_WIDTHx16;	
		h_in : IN  STD_LOGIC_WIDTHx8;	
		s_in : IN  STD_LOGIC_WIDTHx4;	

		hash_out : OUT STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0)
	);
END blake_fin;

ARCHITECTURE blake_fin_a OF blake_fin IS
	SIGNAL hs, V : STD_LOGIC_WIDTHx8;	
	SIGNAL hash_reg, hash_next: STD_LOGIC_VECTOR (WIDTHx8-1 DOWNTO 0);
BEGIN 
	
	PROCESS(clk_in, reset_in)
	BEGIN
		IF (reset_in = '1') THEN
			hash_reg <= (OTHERS => '0');
		ELSIF (clk_in'EVENT AND clk_in = '1') THEN	
			hash_reg <= hash_next;
		END IF;
	END PROCESS;

	hs(0) <= h_in(0) XOR s_in(0);
	hs(1) <= h_in(1) XOR s_in(1);	
	hs(2) <= h_in(2) XOR s_in(2);
	hs(3) <= h_in(3) XOR s_in(3);
	hs(4) <= h_in(4) XOR s_in(0);
	hs(5) <= h_in(5) XOR s_in(1);	
	hs(6) <= h_in(6) XOR s_in(2);	
	hs(7) <= h_in(7) XOR s_in(3);
	
	v(0) <= v_in(0) XOR v_in(8);
	v(1) <= v_in(1) XOR v_in(9);	
	v(2) <= v_in(2) XOR v_in(10);
	v(3) <= v_in(3) XOR v_in(11);
	v(4) <= v_in(4) XOR v_in(12);
	v(5) <= v_in(5) XOR v_in(13);	
	v(6) <= v_in(6) XOR v_in(14);	
	v(7) <= v_in(7) XOR v_in(15);
	
	hash_next(WIDTHx8-1 DOWNTO WIDTHx7)	<= hs(0) XOR v(0); --promenjen raspored zbog ispisa
	hash_next(WIDTHx7-1 DOWNTO WIDTHx6) <= hs(1) XOR v(1);	
	hash_next(WIDTHx6-1 DOWNTO WIDTHx5) <= hs(2) XOR v(2);
	hash_next(WIDTHx5-1 DOWNTO WIDTHx4) <= hs(3) XOR v(3);
	hash_next(WIDTHx4-1 DOWNTO WIDTHx3) <= hs(4) XOR v(4);
	hash_next(WIDTHx3-1 DOWNTO WIDTHx2) <= hs(5) XOR v(5);	
	hash_next(WIDTHx2-1 DOWNTO WIDTHx1) <= hs(6) XOR v(6);	
	hash_next(WIDTHx1-1 DOWNTO 0) 		<= hs(7) XOR v(7);
	
	hash_out <= hash_reg;
	
END ARCHITECTURE;