LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.blake_pak.ALL;

ENTITY round_2_pipe IS
	PORT (
		clk_in : IN STD_LOGIC; 
		reset_in : IN STD_LOGIC;
		
		round_cnt_in : IN  INTEGER;
		
		round_2_h_in : IN STD_LOGIC_WIDTHx8;
		round_2_m_in : IN STD_LOGIC_WIDTHx16;
		round_2_s_in : IN STD_LOGIC_WIDTHx4;	
		round_2_v_in : IN STD_LOGIC_WIDTHx16;
		
		round_2_h_out : OUT STD_LOGIC_WIDTHx8;
		round_2_m_out : OUT STD_LOGIC_WIDTHx16;
		round_2_s_out : OUT STD_LOGIC_WIDTHx4;	
		round_2_v_out : OUT STD_LOGIC_WIDTHx16
	);
END round_2_pipe;

ARCHITECTURE round_2_pipe_a OF round_2_pipe IS
	SIGNAL	h_reg , h_next : STD_LOGIC_WIDTHx8;
	SIGNAL	m_reg , m_next : STD_LOGIC_WIDTHx16;			
	SIGNAL	s_reg , s_next : STD_LOGIC_WIDTHx4;			
	SIGNAL	v_reg , v_next : STD_LOGIC_WIDTHx16;
	SIGNAL	m , c :STD_LOGIC_WIDTHx8;
	
	COMPONENT blake_g IS
		PORT (
			a : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			b : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			c : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			d : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			
			m0_in : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			m1_in : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			con0_in : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			con1_in : IN  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);

			a2 : OUT  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			b2 : OUT  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			c2 : OUT  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0);
			d2 : OUT  STD_LOGIC_VECTOR	(WIDTHx1-1 DOWNTO 0)
		);
	END COMPONENT;	
	
BEGIN 
	PROCESS(clk_in, reset_in)
	BEGIN
		IF (reset_in = '1') THEN
			h_reg <= INIT_WIDTHx8;
			m_reg <= INIT_WIDTHx16;			
			s_reg <= INIT_WIDTHx4;			
			v_reg <= INIT_WIDTHx16;
		ELSIF (clk_in'EVENT AND clk_in = '1') THEN	
			h_reg <= h_next;
			m_reg <= m_next;			
			s_reg <= s_next;			
			v_reg <= v_next;			
		END IF;
	END PROCESS;
	
	m(0)<= round_2_m_in(m_num(round_cnt_in)(8));
	m(1)<= round_2_m_in(m_num(round_cnt_in)(9));
	m(2)<= round_2_m_in(m_num(round_cnt_in)(10));
	m(3)<= round_2_m_in(m_num(round_cnt_in)(11));
	m(4)<= round_2_m_in(m_num(round_cnt_in)(12));
	m(5)<= round_2_m_in(m_num(round_cnt_in)(13));
	m(6)<= round_2_m_in(m_num(round_cnt_in)(14));
	m(7)<= round_2_m_in(m_num(round_cnt_in)(15));
	
	c(0)<= c_con(round_cnt_in)(8);
	c(1)<= c_con(round_cnt_in)(9);
	c(2)<= c_con(round_cnt_in)(10);
	c(3)<= c_con(round_cnt_in)(11);
	c(4)<= c_con(round_cnt_in)(12);
	c(5)<= c_con(round_cnt_in)(13);
	c(6)<= c_con(round_cnt_in)(14);
	c(7)<= c_con(round_cnt_in)(15);
 
	blake_g_com_1: blake_g 
	PORT MAP (
			a => round_2_v_in(0) ,
			b => round_2_v_in(5) ,
			c => round_2_v_in(10) ,
			d => round_2_v_in(15) ,
			
			m0_in => m(0) ,
			m1_in => m(1) ,
			con0_in => c(0) ,
			con1_in => c(1) ,

			a2 =>  v_next(0),
			b2 =>  v_next(5),
			c2 =>  v_next(10),
			d2 =>  v_next(15)
	);

	blake_g_com_2: blake_g 
	PORT MAP (
			a => round_2_v_in(1) ,
			b => round_2_v_in(6) ,
			c => round_2_v_in(11) ,
			d => round_2_v_in(12) ,
			
			m0_in => m(2) ,
			m1_in => m(3) ,
			con0_in => c(2) ,
			con1_in => c(3) ,

			a2 =>  v_next(1),
			b2 =>  v_next(6),
			c2 =>  v_next(11),
			d2 =>  v_next(12)
	);	

	blake_g_com_3: blake_g 
	PORT MAP (
			a => round_2_v_in(2) ,
			b => round_2_v_in(7) ,
			c => round_2_v_in(8) ,
			d => round_2_v_in(13) ,
			
			m0_in => m(4) ,
			m1_in => m(5) ,
			con0_in => c(4) ,
			con1_in => c(5) ,

			a2 =>  v_next(2),
			b2 =>  v_next(7),
			c2 =>  v_next(8),
			d2 =>  v_next(13)
	);


	blake_g_com_4: blake_g 
	PORT MAP (
			a => round_2_v_in(3) ,
			b => round_2_v_in(4) ,
			c => round_2_v_in(9) ,
			d => round_2_v_in(14) ,
			
			m0_in => m(6) ,
			m1_in => m(7) ,
			con0_in => c(6) ,
			con1_in => c(7) ,

			a2 =>  v_next(3),
			b2 =>  v_next(4),
			c2 =>  v_next(9),
			d2 =>  v_next(14)
	);


	h_next <= round_2_h_in;
	m_next <= round_2_m_in;	
	s_next <= round_2_s_in;		

	round_2_h_out <= h_reg;
	round_2_m_out <= m_reg;
	round_2_s_out <= s_reg;
	round_2_v_out <= v_reg;
	
END ARCHITECTURE;