-------------------------------------------------------------------------------
-- File         : BLAKE_PIPE_ROUND_G.vhd
-- Autor        : Goran Ognjanovic
-- Simulator    : ModelSim
-- Sintetizator : Quartus II
-- Datum        : 2012, Avgust 9th
-- Opis modula  : G funkcija, osnovi gradivni blok Round funkcije.
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE work.blake_pak.ALL;

ENTITY blake_g IS
	PORT (
		a : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		b : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		c : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		d : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		
		m0_in   : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);    -- m(2i)
		m1_in   : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);    -- m(2i+1)
		con0_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);    -- c(2i+1)
		con1_in : IN STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);    -- c(2i)

		a2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		b2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		c2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
		d2 : OUT STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0)	
	);
END blake_g;

ARCHITECTURE blake_g_a OF blake_g IS
	SIGNAL t1  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t2  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t3  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t4  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t5  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t6  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t7  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t8  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t9  : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t10 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t11 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0); 
	SIGNAL t12 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t13 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
	SIGNAL t14 : STD_LOGIC_VECTOR(WIDTHx1-1 DOWNTO 0);
BEGIN 

	t1 <= m0_in XOR con0_in;
	t2 <= b+a+t1;                                            --  a<= a + b + (m(2i) XOR c(2i+1))
	t3 <= d XOR t2;
	t4 <= t3(ROT1-1 DOWNTO 0) & t3(WIDTHx1-1 DOWNTO ROT1);	 --  d<= (d XOR a)>> ROT1
	t5 <= c + t4;                                            --  c<= c + d;
	t6 <= b XOR t5;
	t7 <= t6(ROT2-1 DOWNTO 0) & t6(WIDTHx1-1 DOWNTO ROT2);	 --  b<= (b XOR c)>> ROT2
	t8 <= m1_in XOR con1_in;
	t9 <= t2 +t7 +t8 ;                                       --  a<= a + b + (m(2i+1) XOR c(2i))
	t10 <= t4 XOR t9;
	t11 <=t10(ROT3-1 DOWNTO 0) & t10(WIDTHx1-1 DOWNTO ROT3); --  d<= (d XOR a)>> ROT3
	t12 <= t5+t11;                                           --  c<= c + d;
	t13 <= t7 XOR t12;
	t14 <=t13(ROT4-1 DOWNTO 0) & t13(WIDTHx1-1 DOWNTO ROT4); --  b<= (b XOR c)>> ROT4
	
	a2 <= t9;      --a
	d2 <= t11;     --d
	c2 <= t12;     --c
	b2 <= t14;     --b
	
END ARCHITECTURE;