LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE WORK.blade_pak.ALL;

ENTITY testbench_pipe IS
END testbench_pipe;

ARCHITECTURE testbench_pipe_a OF testbench_pipe IS

	COMPONENT blade_pipe IS
		PORT(
			clk : IN STD_LOGIC; 
			reset : IN STD_LOGIC;
			h : IN  STD_LOGIC_WIDTHx8;
			m : IN  STD_LOGIC_WIDTHx16;
			s : IN  STD_LOGIC_WIDTHx4;
			t : IN  STD_LOGIC_WIDTHx2;
			inEN : IN  STD_LOGIC;
			hash  : OUT STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0);
			outEN : OUT  STD_LOGIC
		);
	END COMPONENT;

	SIGNAL test_clk:  std_logic := 'L'; 
	SIGNAL test_reset:  std_logic := 'L';		
	SIGNAL test_h : STD_LOGIC_WIDTHx8 := INIT_WIDTHx8;
	SIGNAL test_m : STD_LOGIC_WIDTHx16:= INIT_WIDTHx16;
	SIGNAL test_s : STD_LOGIC_WIDTHx4 := INIT_WIDTHx4;
	SIGNAL test_t : STD_LOGIC_WIDTHx2 := INIT_WIDTHx2;
	SIGNAL test_inEN :   STD_LOGIC:= 'L';
	SIGNAL test_hash  :  STD_LOGIC_VECTOR (WIDTHx16-1 DOWNTO 0) :=(OTHERS => 'L'); 
	SIGNAL test_outEN :  STD_LOGIC:= 'L';
	
	CONSTANT T : time := 20ms;
	
	TYPE mem_data_type   IS array (0 TO 511) OF STD_LOGIC_VECTOR (WIDTHx1-1 DOWNTO 0);
	SIGNAL mem_pipe : mem_data_type;
	SIGNAL mem_org  : mem_data_type;	
	SIGNAL monitor_num : NATURAL:= 0;
	SIGNAL the_end : STD_LOGIC:= '0';
	SIGNAL error : STD_LOGIC:= '0';
	SIGNAL err_data : STD_LOGIC:= '0';
	
BEGIN 

	blade_pipe_com: blade_pipe
	PORT MAP (
		clk => test_clk ,
		reset => test_reset ,
		h => test_h ,
		m => test_m ,
		s => test_s ,
		t => test_t ,
		inEN => test_inEN ,
		hash => test_hash ,
		outEN => test_outEN 
	);
--****************************************************************************************************
--generatori takta
--****************************************************************************************************
	PROCESS
	BEGIN
		test_clk <= '0';
		WAIT FOR T/2;
		test_clk <= '1';
		WAIT FOR T/2;
	END PROCESS;
	
	test_vp_reset_out  <='1', '0' AFTER T/2;	
	
--****************************************************************************************************	
	PROCESS
	BEGIN
		WAIT UNTIL falling_edge (test_ocp_reset_out);
		WAIT UNTIL falling_edge (test_ocp_clk_out);
		WAIT UNTIL falling_edge (test_ocp_clk_out);
		
	

		
		FOR i IN 0 TO LOOP_NUM-1 LOOP
		--start pix generator 
			WAIT UNTIL falling_edge (test_ocp_clk_out);	
			pix_gen_start<='1';			--start pix generator 
			WAIT UNTIL pix_gen_stop='1';-- pix generator zavrsio
			pix_gen_start<='0';			
			WAIT UNTIL falling_edge (test_ocp_clk_out);	
			
		
		END LOOP;
		
	--wait OCP to finish current frame	
		FOR i IN 0 TO 2**FIFO_DEPTH_CON LOOP 
			WAIT UNTIL falling_edge (test_ocp_clk_out);
		END LOOP;
	--read status
		test_reg_mcmd_out <="010"; 
		test_reg_addr_out <="100"; 
		IF(test_reg_scmd_in /= '1') THEN WAIT ON test_reg_scmd_in; END IF; 
		IF(test_reg_sresp_in /= "01") THEN WAIT ON test_reg_sresp_in; END IF;
		WAIT UNTIL falling_edge (test_ocp_clk_out);
	--init state
		test_reg_mcmd_out <="000";  
		test_reg_addr_out <="000"; 
		test_reg_data_out <=x"00000000";
		WAIT UNTIL falling_edge (test_ocp_clk_out);			
		
		ststus_out<='1'; --pokrecemo proveru statusa
		WAIT UNTIL falling_edge (test_ocp_clk_out);
		the_end<='1';		--oznacavamo da je kraj
		WAIT UNTIL test_ocp_reset_out='1';	--beskonacno cekamo
		
	END PROCESS;
		
--****************************************************************************************************
--pix vektor generator
--****************************************************************************************************
	PROCESS
		--random logic
		VARIABLE seed1, seed2: POSITIVE; 
		VARIABLE rand: REAL;  
		VARIABLE int_rand: INTEGER; 
		
		VARIABLE pix_per_line_gen : INTEGER := 0; 
		VARIABLE line_num_gen : INTEGER := 0;
		VARIABLE pix_error : INTEGER := 0;
		VARIABLE line_error : INTEGER := 0;
	BEGIN	
		WAIT UNTIL pix_gen_start='1';			--cekamo na signalizaciju
		WAIT UNTIL falling_edge (test_vp_clk_out);
		
		test_vp_data_valid_out <= '0' ;
		test_vp_data_out <= "00000000" ; 
		test_vp_hs_out <= '0' ; 	
		test_vp_he_out <= '0' ; 
		test_vp_vs_out <= '0' ; 
		test_vp_ve_out <= '0' ; 	
		
		WAIT UNTIL falling_edge (test_vp_clk_out);
		
		
		
		line_num_gen := line_num;		
		FOR i IN 1 TO line_num_gen LOOP--na osnovu predhodno dobijenih dimenzija matrice pixela pokrece sa dvostruka for petlja, i vrsta,j kolona
	
			pix_per_line_gen := pix_per_line ;
			FOR j IN 1 TO pix_per_line_gen LOOP
			
				WAIT UNTIL falling_edge (test_vp_clk_out);
				IF(test_vp_stall_in = '1') THEN	--ako je stall aktivan cekamo dok ne padne na 0,i na sledecu ivicu takta nastavljamo sa generisanjem test vektora
					test_vp_data_valid_out <= '0' ;
					WAIT ON test_vp_stall_in; 
					WAIT UNTIL falling_edge (test_vp_clk_out); 
				END IF; 	
				test_vp_data_valid_out <= '1' ; 
				test_vp_data_out <= test_vp_data_out+1; --osmo bitna vrednost pixala,nije bitna,osim za pracenje ispravne sekvence slanja 
					
				IF(i=1 AND j=1) THEN --vs prvi pixel nase slike
					test_vp_hs_out <= '0' ; 	
					test_vp_he_out <= '0' ; 
					test_vp_vs_out <= '1' ; 
					test_vp_ve_out <= '0' ; 	
					
				ELSIF(i=line_num_gen AND j=pix_per_line_gen) THEN --ve poslednji pixel nase slike
					test_vp_hs_out <= '0' ; 	
					test_vp_he_out <= '0' ; 
					test_vp_vs_out <= '0' ; 
					test_vp_ve_out <= '1' ; 
					
				ELSIF(i/=1 AND j=1) THEN --hs prvi pixel linije nase slike
					test_vp_hs_out <= '1' ; 	
					test_vp_he_out <= '0' ; 
					test_vp_vs_out <= '0' ; 
					test_vp_ve_out <= '0' ; 
					
				ELSIF(i/=line_num_gen AND j=pix_per_line_gen) THEN--he poslednji pixel linije nase slike
					test_vp_hs_out <= '0' ; 	
					test_vp_he_out <= '1' ; 
					test_vp_vs_out <= '0' ; 
					test_vp_ve_out <= '0' ; 
					
				ELSE --ostali pixeli
					test_vp_hs_out <= '0' ; 	
					test_vp_he_out <= '0' ; 
					test_vp_vs_out <= '0' ; 
					test_vp_ve_out <= '0' ; 					
				END IF;
		
			END LOOP;
			
		END LOOP;
		
		
		WAIT UNTIL falling_edge (test_vp_clk_out);
		
		test_vp_data_valid_out <= '0' ;
		test_vp_data_out <= "00000000" ; 
		test_vp_hs_out <= '1'; 	
		test_vp_he_out <= '1'; 
		test_vp_vs_out <= '1'; 
		test_vp_ve_out <= '1'; 	
		
		WAIT UNTIL falling_edge (test_vp_clk_out);
		
		
		WAIT UNTIL falling_edge (test_ocp_clk_out);
		pix_gen_stop<='1';								--signaliziramo kraj
		WAIT UNTIL falling_edge (test_ocp_clk_out);
		pix_gen_stop<='0';	
		WAIT UNTIL falling_edge (test_vp_clk_out);
	END PROCESS;

--****************************************************************************************************
--monitor
--****************************************************************************************************
	PROCESS (test_ocp_clk_out, test_ocp_reset_out)
	BEGIN

		IF (test_ocp_reset_out = '1') THEN
			monitor_num<= 0;
		ELSIF (test_ocp_clk_out'EVENT and test_ocp_clk_out = '1') THEN
			
			IF (test_ocp_mcmd_in = "001")AND(test_ocp_scmd_accept_out = '1') THEN --kada je ocp master postavio mcmd  i ocp slave odgovorio sam ocp_scmd_accept='1',to je trenuta kada mozemo izlaze da poredimo sa predvidjenim vrednostima
			
				ASSERT not(test_ocp_maddr_in /= mem_addr(monitor_num)) 
					REPORT "address error"				
				SEVERITY FAILURE;
				
				ASSERT not(test_ocp_mdata_in /= mem_data(monitor_num)) 
					REPORT "data error"				
				SEVERITY FAILURE;
				
				ASSERT not(test_ocp_mdata_byte_en_in /= mem_byte(monitor_num))
					REPORT "byte enable error"
				SEVERITY FAILURE;
   
				monitor_num<=monitor_num+1;-- inkrementiramo poziciju u memoriji sa kojom poredimo izlaz
			END IF;

			ASSERT not(ststus_out='1' AND test_reg_sdata_in /= mem_status) -- status se proverava na kraju
				REPORT "status error"									
			SEVERITY FAILURE;
			
		END IF;	
	END PROCESS;
	
--****************************************************************************************************
--the end
--****************************************************************************************************	
	PROCESS -- kraj
	BEGIN
		WAIT UNTIL the_end='1';
		
		ASSERT false
			REPORT "simulation completed"	
		SEVERITY NOTE;
		
	END PROCESS;
	
END ARCHITECTURE ;